-- ****************************************************************************
-- * Licensed Materials - Property of IBM
-- * 5724-T40 IBM License Metric Tool
-- * 5724-S94 IBM Tivoli Asset Discovery for Distributed
-- *
-- * (C) Copyright IBM Corp. 2012.  All Rights Reserved.
-- *
-- * US Government Users Restricted Rights - Use, duplication or
-- * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
-- ****************************************************************************

connect to tlma%

create procedure ADM.DROP_IF_EXISTS(in NAME char(100))
	begin
	declare STMT char(250);
	declare MSG char(20);
	
	declare continue handler for sqlstate '42704' set MSG = 'ok';
	
	set STMT = 'DROP ' concat NAME;
	prepare S1 from STMT;
	execute S1;
	end%

-- create a temp table to save existing select grants for the view
call ADM.DROP_IF_EXISTS('table ADM.FP_SAVE_GRANTS')%
create table ADM.FP_SAVE_GRANTS
(
   grant_cmd varchar(250)
)%

create procedure ADM.RESTORE_VIEW_ACCESS()
   begin
    declare STMT char(250);
    declare prep_stmt char(250);

    for v1 as c1 cursor for select grant_cmd from ADM.FP_SAVE_GRANTS
      do
         set STMT= grant_cmd;
         prepare prep_stmt from STMT;
         execute prep_stmt;
      end for;
   end%
   
   	
insert into adm.fp_save_grants (grant_cmd) select 'grant select on CUSTOM.AGENT_V to user ' || grantee from sysibm.systabauth where tcreator='CUSTOM' and ttname='AGENT_V' and selectauth in ('Y','G')%

call ADM.DROP_IF_EXISTS('view CUSTOM.AGENT_V')%

create view CUSTOM.AGENT_V
(
	ID,
	HOSTNAME,
	IP_ADDRESS,
	VERSION,
	OS_NAME,
	OS_VERSION,
	PLUGIN_TIME,
	SCAN_TIME,
	LOCAL_SCAN_TIME,
	ACTIVE,
	NODE_ID,
	SCAN_GROUP_ID,
	SECURITY_LEVEL,
	INV_SYNCED,
	CUSTOM_DATA1,
	CUSTOM_DATA2,
	CUSTOM_DATA3,
	FULL_HWSCAN_TIME,
	FULL_HWSCAN_LOCAL_TIME,
	CATALOG_VERSION,
	DELETED_TIME,
	PACKET_IP_ADDRESS,
	STATUS
)
as
  select
	A.ID,
	A.HOSTNAME,
	A.IP_ADDRESS,
	A.VERSION,
	A.OS_NAME,
	A.OS_VERSION,
	A.PLUGIN_TIME,
	A.SCAN_TIME,
	A.LOCAL_SCAN_TIME,
	A.ACTIVE,
	A.NODE_ID,
	A.DIVISION_ID,
	A.SECURITY_LEVEL,
	A.INV_SYNCED,
	A.CUSTOM_DATA1,
	A.CUSTOM_DATA2,
	A.CUSTOM_DATA3,
	A.HWSCAN_SERVER_TIME,
	A.HWSCAN_TIME,
	A.CATALOG_TIME,
    cast(null as timestamp),
    A.PACKET_IP_ADDRESS,
    STATUS.STATUS
    from
     ADM.AGENT A
     JOIN (
	     SELECT 
	     agent.id AS agent_id, 
	     CASE 
	       WHEN  
	           agent.active = 2 --ACTIVE_UNKNOWN
	       THEN  
	 	     5  --STATUS_UNKNOWN
	       WHEN  
	     	   agent.node_id IS NULL AND 
				   CURRENT TIMESTAMP - CURRENT TIMEZONE - 60 SECONDS <= agent.plugin_time   
	       THEN  
	           2 --STATUS_INITIALIZING
	       WHEN  --inactive status does not apply to standalone agents
	     	  division.type = 0 --TYPE_STANDARD  
	           AND (agent_alive.alive_time is NULL or agent_alive.alive_time < 
			    (bigint((DAYS(current timestamp - current timezone) - DAYS('1970-01-01-00.00.00.000000')) * 86400 + MIDNIGHT_SECONDS(current timestamp - current timezone)))*1000 + MICROSECOND(CURRENT TIMESTAMP)/1000
				 - (cast((select control_value from lic.control_values where control_key like 'maxAgentInactivity') as bigint) * 60 * 1000)
			   )
	       THEN  
	           3 --STATUS_NOT_CONNECTING
	       WHEN  
	           agent.status <>  0 --AGENT_STATUS_NO_ERROR  
	       THEN  
	 	     4 --STATUS_FAILED		
	       WHEN  
	           agent.node_id IS NULL AND 
				  CURRENT TIMESTAMP - CURRENT TIMEZONE - 60 SECONDS > agent.plugin_time 
	       THEN  
	 	     8 --STATUS_MISSING_HW_SCAN		
	       WHEN  
	          (agent_vm.end_time IS NOT NULL AND agent_vm.end_time = '9999-12-31 00:00:00.000000') OR 
	          (agent_node.tag IS NOT NULL AND agent_node.tag LIKE 'TLM_VM%') 
	       THEN  
	          6 --STATUS_INCOMPLETE 		
	       WHEN  
	          agent_scan_status.sw_scan_status = 2 --STATUS_FAILED
	       THEN  
	         7 --STATUS_MISSING_SW_SCAN		
	       ELSE  
	          1 --STATUS_OK
	     END AS status
	   FROM 
	     adm.agent agent 
	   LEFT JOIN  
	     adm.division division ON agent.division_id=division.id 
	   INNER JOIN 
	     adm.agent_scan_status agent_scan_status ON agent.id = agent_scan_status.agent_id 
	   LEFT OUTER JOIN  
	     adm.agent_alive agent_alive ON agent.id=agent_alive.agent_id 
	   LEFT OUTER JOIN 
	    ( SELECT  
	        agent_id AS agent_id, 
			   MAX(end_time) AS end_time 
		     FROM  
				adm.agt_partial_vm 
		     GROUP BY 
			    agent_id 
	    ) agent_vm  ON agent.id=agent_vm.agent_id 
	    LEFT OUTER JOIN 
	      adm.node agent_node ON agent.node_id=agent_node.id
    ) as status on status.agent_id = a.id
    union all
    select
	A.ID,
	A.HOSTNAME,
	A.IP_ADDRESS,
	A.VERSION,
	A.OS_NAME,
	A.OS_VERSION,
	A.PLUGIN_TIME,
	A.SCAN_TIME,
	CAST(null as timestamp),
	0,
	A.NODE_ID,
	A.DIVISION_ID,
	A.SECURITY_LEVEL,
	CAST(null as integer),
	A.CUSTOM_DATA1,
	A.CUSTOM_DATA2,
	A.CUSTOM_DATA3,
	CAST(null as timestamp),
	CAST(null as timestamp),
	A.CATALOG_TIME,
	A.DELETED_TIME,
	CAST(null as varchar(39)),
	5
    from
      ADM.AGENT_DELETED A%



comment on table CUSTOM.AGENT_V is 'Agents which are deployed with this server instance'%
comment on column CUSTOM.AGENT_V.HOSTNAME is 'System host name (e.g. omega.ibm.com)'%
comment on column CUSTOM.AGENT_V.IP_ADDRESS is 'Network IP address'%
comment on column CUSTOM.AGENT_V.VERSION is 'Version of the agent that is installed'%
comment on column CUSTOM.AGENT_V.OS_NAME is 'Operating system name (Unix, Windows, OS/400)'%
comment on column CUSTOM.AGENT_V.OS_VERSION is 'Operating system version (e.g. 2.1, NT4, 2000)'%
comment on column CUSTOM.AGENT_V.PLUGIN_TIME is 'Date and time of the last agent plug-in or update'%
comment on column CUSTOM.AGENT_V.SCAN_TIME is 'Date and time of the last available software scan'%
comment on column CUSTOM.AGENT_V.LOCAL_SCAN_TIME is 'Agent local date and time of the last available software scan'%
comment on column CUSTOM.AGENT_V.ACTIVE is 'If true, the agent is active'%
comment on column CUSTOM.AGENT_V.NODE_ID is 'External key to NODE_V view'%
comment on column CUSTOM.AGENT_V.SCAN_GROUP_ID is 'External key to SCAN_GROUP_V'%
comment on column CUSTOM.AGENT_V.SECURITY_LEVEL is '0=HTTP; 1=HTTPS (server authentication); 2=HTTPS (server & agent authentication)'%
comment on column CUSTOM.AGENT_V.INV_SYNCED is '0 = Inventory out of sync; 1 = Inventory synced.'%
comment on column CUSTOM.AGENT_V.CUSTOM_DATA1 is 'Custom data'%
comment on column CUSTOM.AGENT_V.CUSTOM_DATA2 is 'Custom data'%
comment on column CUSTOM.AGENT_V.CUSTOM_DATA3 is 'Custom data'%
comment on column CUSTOM.AGENT_V.FULL_HWSCAN_TIME is 'Date and time of the last available full hw scan'%
comment on column CUSTOM.AGENT_V.FULL_HWSCAN_LOCAL_TIME is 'Agent local date and time of the last available full hw scan'%
comment on column CUSTOM.AGENT_V.CATALOG_VERSION is 'Date and time of the last catalog update'%
comment on column CUSTOM.AGENT_V.DELETED_TIME is 'Timestamp of agents deletion'%
comment on column CUSTOM.AGENT_V.ID is 'Synthetic identifier'%
comment on column CUSTOM.AGENT_V.PACKET_IP_ADDRESS is 'Source IP address of last received packet. It can be address of agent or some proxy between agent and server.'%
comment on column CUSTOM.AGENT_V.STATUS is 'Status of agent: 1 = ok, 2 = initializing, 3 = not connecting, 4 = failed, 5 = unknown, 6 = incomplete, 7 = missing sw scan, 8 = missing hw scan'%

-- clean up 
call ADM.RESTORE_VIEW_ACCESS%
call ADM.DROP_IF_EXISTS('table ADM.FP_SAVE_GRANTS')%

drop procedure ADM.DROP_IF_EXISTS%
drop procedure ADM.RESTORE_VIEW_ACCESS%

disconnect current%
	